# -*- coding: utf-8 -*-
from scrapy.spiders import CrawlSpider, Rule
from scrapy.linkextractors import LinkExtractor


class agapiSpider(CrawlSpider):
    name = "agentura-api"

    allowed_domains = ["www.agentura-api.org"]

    start_urls = [
        "https://www.agentura-api.org/programy-podpory/",
    ]

    rules = [
        Rule(LinkExtractor(r'programy-podpory\/.*vyzva.*'), callback='vyzva', follow=True),
        Rule(LinkExtractor(), follow=True),
    ]

    def vyzva(self, response):
        vyzva = {}
        vyzva['zdroj'] = response.url
        vyzva['titul'] = response.xpath('//h1[@class="entry-title"]//text()').get()

        meta = response.xpath('//div[@class="boxer open-toggler"]')
        v_stav = meta.xpath('.//div[contains(@class, "boxer-header")]//text()').getall()
        vyzva['stav'] = ''.join(map(lambda s: self.t_strip(s), v_stav))
        if vyzva['stav'] == "Uzavřená výzva":
            return None

        for m_item in meta.xpath('.//div[contains(@class, "boxer-item")]'):
            atribut = ''.join(map(lambda s: self.t_strip(s), m_item.xpath('.//span[1]//text()').getall()))
            hodnota = ''.join(map(lambda s: self.t_strip(s), m_item.xpath('.//span[2]//text()').getall()))
            vyzva[atribut] = hodnota

        content = response.xpath('//div[@class="entry-content content-area"]')
        nadpisy = 'p[count(strong)>0]'
        count_nadpisy = int(float(content.xpath('count(' + nadpisy + ')').get()))
        for idx in range(1, count_nadpisy + 1):
            prec_s = 'preceding-sibling::' + nadpisy + '[' + str(idx) + ']'
            foll_s = 'following-sibling::' + nadpisy + '[' + str(count_nadpisy - idx) + ']'

            atribut = ''.join(map(lambda s: self.t_strip(s),
                                  content.xpath(nadpisy + '[' + str(idx) + ']' + '//text()').getall()))
            polozky = []
            if idx == 1:    # prvni v poradi je popis a harmonogram
                vyzva['popis'] = atribut
                polozky = content.xpath('*[self::p][' + prec_s + ' and ' + foll_s + ']//text()').getall()
                vyzva['harmonogram'] = ''.join(map(lambda s: self.t_strip(s), polozky))
            else:   # nasleduje seznam charakteristik
                if idx == count_nadpisy:    # posledni v poradi
                    polozky = content.xpath('*[self::ul/li][' + prec_s + ']//text()').getall()
                else:
                    polozky = content.xpath('*[self::ul/li][' + prec_s + ' and ' + foll_s + ']//text()').getall()
                hodnoty = list(map(lambda s: self.t_strip(s), polozky))
                vyzva[atribut] = list(filter(lambda s: len(s) > 0, hodnoty))

        yield vyzva

    def t_strip(self, s):
        text = s.replace('\n', '').replace('\t', '').replace('\r', '')
        text = text.strip()
        return text
